package com.example.es8;

import co.elastic.clients.elasticsearch.ElasticsearchClient;
import co.elastic.clients.elasticsearch.core.*;
import co.elastic.clients.elasticsearch.core.bulk.BulkOperation;
import co.elastic.clients.elasticsearch.core.search.Hit;
import io.jsonwebtoken.lang.Maps;
import lombok.extern.slf4j.Slf4j;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.test.context.ActiveProfiles;

import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 测试 elasticSearch client 文档相关的操作
 *
 * @author WT
 * @date 2021/12/28 16:27:45
 */
@Slf4j
@SpringBootTest
@ActiveProfiles("wr") //这里
public class ElasticsearchClientDocumentTest {

    private static final String INDEX_NAME = "elasticsearch-client";
    @Autowired
    private ElasticsearchClient elasticsearchClient;

    // 添加文档
    @Test
    public void testAddDocument() throws IOException {

        IndexResponse indexResponse = elasticsearchClient.index(indexRequest -> {
//                indexRequest.index(INDEX_NAME).document(new AhaIndex().setName("wangWu").setAge(21))
//                        indexRequest.index(INDEX_NAME).document(new DocsDTO("wangrui",18))
                    HashMap<Object, Object> map = new HashMap<>();
                    map.put("name2", "zhangsan2");
                    map.put("age3", "23");
                    return indexRequest.index(INDEX_NAME).document(map);
                }
        );
        log.info("== response: {}, responseStatus: {}", indexResponse, indexResponse.result());

    }

    // 获取文档信息
    @Test
    public void testGetDocument() throws IOException {
        GetResponse<Map> getResponse = elasticsearchClient.get(getRequest ->
                getRequest.index(INDEX_NAME).id("1"), Map.class
        );
        log.info("== document source: {}, response: {}", getResponse.source(), getResponse);
    }

    // 更新文档信息
    @Test
    public void testUpdateDocument() throws IOException {
        UpdateResponse<Map> updateResponse = elasticsearchClient.update(updateRequest ->
                        updateRequest.index(INDEX_NAME).id("tU4YAH4B395pyiY3b46F")
//                        .doc(new AhaIndex().setName("lisi1").setAge(22)), Map.class
                                .doc(Maps.of("name", "wangrui2")), Map.class
        );
        log.info("== response: {}, responseStatus: {}", updateResponse, updateResponse.result());
    }

    // 删除文档信息
    @Test
    public void testDeleteDocument() throws IOException {
        DeleteResponse deleteResponse = elasticsearchClient.delete(deleteRequest ->
                deleteRequest.index(INDEX_NAME).id("1")
        );
        log.info("== response: {}, result:{}", deleteResponse, deleteResponse.result());

    }

    // 批量插入文档
    @Test
    public void testBatchInsert() throws IOException {

        List<BulkOperation> bulkOperationList = new ArrayList<>();

        for (int i = 0; i < 10; i++) {
//            AhaIndex ahaIndex = new AhaIndex().setName("lisi" + i).setAge(20 + i);
            int finalI = i;
            bulkOperationList.add(new BulkOperation.Builder().create(e -> e.document(Maps.of("name", "wangrui" + finalI))).build());
        }

        BulkResponse bulkResponse = elasticsearchClient.bulk(bulkRequest ->
                bulkRequest.index(INDEX_NAME).operations(bulkOperationList)
        );

        // 这边插入成功的话显示的是 false
        log.info("== errors: {}", bulkResponse.errors());
    }


    @Test
    public void testRestClient() throws IOException {

        SearchResponse<Map> search = elasticsearchClient.search(s -> s.index("aha-batch")
                        .query(q -> q.term(t -> t
                                .field("name")
                                .value(v -> v.stringValue("1aha"))
                        )),
                Map.class);

        for (Hit<Map> hit : search.hits().hits()) {
            log.info("== hit: {}", hit.source());
        }

    }

}
