package com.search.supersearch.controller;

import co.elastic.clients.elasticsearch.ElasticsearchClient;
import co.elastic.clients.elasticsearch.core.CountRequest;
import co.elastic.clients.elasticsearch.core.CountResponse;
import co.elastic.clients.elasticsearch.core.SearchRequest;
import co.elastic.clients.elasticsearch.core.SearchResponse;
import co.elastic.clients.elasticsearch.core.search.Hit;
import co.elastic.clients.elasticsearch.core.search.HitsMetadata;
import com.google.common.collect.Lists;
import com.search.supersearch.common.R;
import com.search.supersearch.config.ElasticSearchConfig;
import com.search.supersearch.dto.SearchDTO;
import com.search.supersearch.pojo.DocsIndex;
import com.search.supersearch.vo.DocsIndexVO;
import lombok.SneakyThrows;
import lombok.extern.slf4j.Slf4j;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;

@RestController
@RequestMapping("/docs")
@Slf4j
public class DocsController {

    @Resource
    private ElasticsearchClient elasticsearchClient;
    @Resource
    private ElasticSearchConfig elasticSearchConfig;

    private List<String> es_index = Lists.newArrayList("001001", "002001", "002002", "003001", "003002", "003003");

    @PostMapping("/search")
    @SneakyThrows
    public R<?> search(@RequestBody SearchDTO dto) {

//        List<SearchType> searchType = FileUtil.getSearchType(elasticSearchConfig.getIndex());
//        ArrayList<String> strings = Lists.newArrayList();
//        dto.getSearchType().stream().forEach(type -> {
//            Objects.requireNonNull(searchType.stream().filter(v2 -> v2.getType().equals(type)).findAny().orElse(null))
//                    .getIndexName().stream().forEach(strings::add);
//        });
//
//        List<String> collect = strings.stream().distinct().collect(Collectors.toList());
//        String[] array = new String[collect.size()];
//        collect.toArray(array);
        String keyword = dto.getKeyword();
        if (dto.getSearchType() == null || dto.getSearchType().size() == 0) {
            return R.fail("参数不可谓空!");
        }

        SearchResponse<DocsIndex> search = elasticsearchClient.search(s -> {
                    SearchRequest.Builder index = s.index(dto.getSearchType());
                    if (dto.getMatch() != null && dto.getMatch()) {
                        index.query(q -> q.bool(b -> b
                                .should(s1 -> s1.term(v -> v.field("content").value(keyword)))
                                .should(s2 -> s2.term(v -> v.field("filename").value(keyword)))
                        ));
                    } else {
                        index.query(q -> q.bool(b -> b
                                .should(s1 -> s1.match(v -> v.field("content").query(keyword)))
                                .should(s2 -> s2.match(v -> v.field("filename").query(keyword)))
                        ));
                    }

                    return index.highlight(h -> h.fields("content", hf -> hf).fields("filename", hf -> hf))
                            .size(dto.getSize() == null ? 10 : dto.getSize())
                            .from(dto.getPage() == null ? 0 : dto.getPage() - 1);
                }
                ,
                DocsIndex.class);
        ArrayList<DocsIndexVO> list = Lists.newArrayList();

        HitsMetadata<DocsIndex> hits = search.hits();

        for (Hit<DocsIndex> docsIndexHit : hits.hits()) {
            DocsIndexVO docsIndexVO = new DocsIndexVO();
            docsIndexVO.setSource(docsIndexHit.source());
            docsIndexVO.setHighlight(docsIndexHit.highlight());
            list.add(docsIndexVO);
        }
        return R.data(list);
    }

    @PostMapping("/searchCount")
    @SneakyThrows
    public R<?> searchCount(@RequestBody SearchDTO dto) {

        String keyword = dto.getKeyword();
        if (dto.getSearchType() == null || dto.getSearchType().size() == 0) {
            return R.fail("参数不可谓空!");
        }

        //todo 查询所有二级分类数量
        CountResponse count = elasticsearchClient.count(c -> {
            CountRequest.Builder index = c.index(dto.getSearchType());
            if (dto.getMatch() != null && dto.getMatch()) {
                index.query(q -> q.bool(b -> b
                        .should(s1 -> s1.term(v -> v.field("content").value(keyword)))
                        .should(s2 -> s2.term(v -> v.field("filename").value(keyword)))
                ));
            } else {
                index.query(q -> q.bool(b -> b
                        .should(s1 -> s1.match(v -> v.field("content").query(keyword)))
                        .should(s2 -> s2.match(v -> v.field("filename").query(keyword)))
                ));
            }
            return index;
        });


        return R.data(count.count());
    }

    @PostMapping("/searchCountTotal")
    @SneakyThrows
    public R<?> searchCountTotal(@RequestBody SearchDTO dto) {

        String keyword = dto.getKeyword();

        CountResponse count = elasticsearchClient.count(c -> {
            CountRequest.Builder index = c.index(es_index);
            if (dto.getMatch() != null && dto.getMatch()) {
                index.query(q -> q.bool(b -> b
                        .should(s1 -> s1.term(v -> v.field("content").value(keyword)))
                        .should(s2 -> s2.term(v -> v.field("filename").value(keyword)))
                ));
            } else {
                index.query(q -> q.bool(b -> b
                        .should(s1 -> s1.match(v -> v.field("content").query(keyword)))
                        .should(s2 -> s2.match(v -> v.field("filename").query(keyword)))
                ));
            }
            return index;
        });

        return R.data(count.count());
    }
}
