package com.search.supersearch.controller;

import com.aliyun.oss.OSS;
import com.aliyun.oss.model.Bucket;
import com.aliyun.oss.model.OSSObject;
import com.aliyun.oss.model.OSSObjectSummary;
import com.aliyun.oss.model.ObjectListing;
import com.search.supersearch.common.R;
import com.search.supersearch.config.OSSConfig;
import com.search.supersearch.service.EsService;
import com.search.supersearch.util.AliOssUtil;
import lombok.SneakyThrows;
import lombok.extern.slf4j.Slf4j;
import org.apache.tika.Tika;
import org.apache.tika.metadata.Metadata;
import org.apache.tika.parser.ParseContext;
import org.apache.tika.parser.odf.OpenDocumentParser;
import org.apache.tika.sax.BodyContentHandler;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import java.io.*;
import java.net.URL;
import java.util.Date;
import java.util.List;

@RestController
@RequestMapping("/test/oss")
@Slf4j
public class OssController {
    @Resource
    OSSConfig ossConfig;

    @Resource
    private EsService esService;

    @GetMapping("/list")
    @SneakyThrows
    public R<?> list(String bucketName) {
        OSS oss = ossConfig.restOSSClient();
        ObjectListing objectListing = oss.listObjects(bucketName);
        for (OSSObjectSummary objectSummary : objectListing.getObjectSummaries()) {
            log.info(" - " + objectSummary.getKey() + "  " +
                    "(size = " + objectSummary.getSize() + ")");
        }
        oss.shutdown();
        return R.data(objectListing);
    }

    @GetMapping("/get")
    @SneakyThrows
    public R<?> get(String bucketName, String objectName) {
        OSS oss = ossConfig.restOSSClient();
        // 调用ossClient.getObject返回一个OSSObject实例，该实例包含文件内容及文件元信息。
        OSSObject ossObject = oss.getObject(bucketName, objectName);
        // 调用ossObject.getObjectContent获取文件输入流，可读取此输入流获取其内容。
        InputStream content = ossObject.getObjectContent();
        if (content != null) {
            BufferedReader reader = new BufferedReader(new InputStreamReader(content));
            while (true) {
                String line = reader.readLine();
                if (line == null) {
                    break;
                }
                log.info("\n" + line);
            }
            // 数据读取完成后，获取的流必须关闭，否则会造成连接泄漏，导致请求无连接可用，程序无法正常工作。
            content.close();
        }
        oss.shutdown();
        return R.data(ossObject);
    }

    @GetMapping("/list2")
    @SneakyThrows
    public R<?> list2() {
        List<Bucket> bucketsAll = AliOssUtil.getBucketsAll();
        log.info("bucketsAll,{}", bucketsAll);
        List<OSSObjectSummary> summarysAll = AliOssUtil.getSummarysAll();
        log.info("summarysAll,{}", summarysAll);
        return R.data(summarysAll);
    }

    public static void main(String[] args) {
        System.out.println(new Date());
    }

    @GetMapping("/get/tika")
    @SneakyThrows
    public R<?> gettika(String bucketName, String objectName, String index) {
        OSS oss = ossConfig.restOSSClient();
        // 调用ossClient.getObject返回一个OSSObject实例，该实例包含文件内容及文件元信息。
        OSSObject ossObject = oss.getObject(bucketName, objectName);
        // 设置签名URL过期时间，单位为毫秒。本示例以设置过期时间为1小时为例。
        Date expiration = new Date(System.currentTimeMillis() + 3600 * 1000L);
        // 生成以GET方法访问的签名URL，访客可以直接通过浏览器访问相关内容。
        URL url = oss.generatePresignedUrl(bucketName, objectName, expiration);
        log.info("文件访问 url={}", url);

        String s = esService.saveFileByUrl(url, index);
        log.info("文件内容 ={}", s);
        return R.data(s);

    }


    @GetMapping("/get/tika2")
    @SneakyThrows
    public R<?> gettika2(String bucketName, String objectName) {
        OSS oss = ossConfig.restOSSClient();
        // 调用ossClient.getObject返回一个OSSObject实例，该实例包含文件内容及文件元信息。
        OSSObject ossObject = oss.getObject(bucketName, objectName);
        // 设置签名URL过期时间，单位为毫秒。本示例以设置过期时间为1小时为例。
        Date expiration = new Date(System.currentTimeMillis() + 3600 * 1000L);
        // 生成以GET方法访问的签名URL，访客可以直接通过浏览器访问相关内容。
        URL url = oss.generatePresignedUrl(bucketName, objectName, expiration);
        System.out.println(url);

        Tika tika = new Tika();
        File file = new File("/Users/wangrui/Downloads/2020国家电投春招,待遇怎么样？.docx");
//        String s = tika.parseToString();
        OpenDocumentParser parser = new OpenDocumentParser();

        // 创建 BodyContentHandler 对象
        BodyContentHandler handler = new BodyContentHandler();

        // 创建 Metadata 对象
        Metadata metadata = new Metadata();

        // 创建 ParseContext 对象
        ParseContext context = new ParseContext();

        // 使用 OfficeParser 解析文件
        InputStream stream = new FileInputStream(file);
        parser.parse(stream, handler, metadata, context);

        // 输出文件内容
        System.out.println(handler.toString());
        return R.data(handler.toString());
    }
}
