package com.search.supersearch.controller;

import cn.hutool.core.util.StrUtil;
import cn.hutool.crypto.digest.DigestUtil;
import cn.hutool.http.HttpRequest;
import cn.hutool.http.HttpUtil;
import cn.hutool.json.JSONObject;
import cn.hutool.json.JSONUtil;
import com.search.supersearch.common.R;
import com.search.supersearch.config.OSSConfig;
import com.search.supersearch.model.FileInfo;
import com.search.supersearch.model.PreviewInfo;
import com.search.supersearch.model.UserInfo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.SneakyThrows;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import java.io.IOException;
import java.net.URL;
import java.net.URLConnection;
import java.net.URLStreamHandler;
import java.time.ZoneId;
import java.time.ZonedDateTime;
import java.time.format.DateTimeFormatter;
import java.util.HashMap;
import java.util.Map;
import java.util.Optional;
import java.util.UUID;

@RestController
@RequestMapping("/wps")
@Slf4j
@Api(value = "wps数据预览", tags = "wps数据预览接口")
public class WPSController {
    @Resource
    OSSConfig ossConfig;

    @Value("${wps.api.url}")
    private String url;

    @Value("${wps.appId}")
    private String appId;

    @Value("${wps.appKey}")
    private String appKey;

    @GetMapping("/preview/callback")
    @SneakyThrows
    @ApiOperation("回调地址")
    public JSONObject gettika2(@RequestParam(required = false) Map<String, Object> params) {
        log.info("回调参数："+params);
        PreviewInfo previewInfo = new PreviewInfo();
        FileInfo file = new FileInfo();
        UserInfo user = new UserInfo();
        file.setId("132aa30a87064");
        file.setName("仓库租赁合同.docx");
        file.setVersion(1);
        file.setSize(12297);
        file.setCreator("id0");
        file.setCreateTime(1136185445L);
        file.setModifier("id1000");
        file.setModifyTime(1551409818L);
        file.setDownloadUrl("http://10.104.32.31:9999/download/001001/仓库租赁合同.docx");
        file.setPreview_pages(3);
        user.setId("id1000");
        user.setName("wps-1000");
        user.setPermission("read");
        user.setAvatarUrl("http://xxx.cn/id=1000");
        previewInfo.setFile(file);
        previewInfo.setUserInfo(user);
        JSONObject json = new JSONObject();
        JSONObject data = JSONUtil.parseObj(previewInfo);
        json.set("code",200);
        json.set("data",data);
        // 如果报错则返回一下接口
//        {
//            "result": int64,       //必填,值选取下表
//                "msg": "string",       //必填，出错信息
//        }
        log.info("返回的数据："+json);
        return json;
    }

    /**
     * 获取预览 url
     */
    @GetMapping("/preview/url")
    @ApiOperation("获取wps数据预览url")
    public R<?> getPreviewUrl(@RequestParam(required = false) Map<String,Object> param){
        String appToken = getToken();
        if (StrUtil.isEmpty(appToken)) {
            return R.fail("appToken 为空");
        }
        // TODO 从数据库获取文件 id
        String fileId = "132aa30a87064";
        String body = null;
        String sceneId = UUID.randomUUID().toString().replace("-","");
        String contentMd5 = DigestUtil.md5Hex(Optional.<String>ofNullable(body).orElse(""));
        String dateStr = ZonedDateTime.now().format(DateTimeFormatter.RFC_1123_DATE_TIME.withZone(ZoneId.of("GMT")));
        String urls = "/preview/v2/url?app_token=" + appToken +"&file_id=" + fileId + "&scene_id=" + sceneId;
        // 处理请求头
        String xAuth = String.format("WPS-3:%s:%s", appId, DigestUtil.sha1Hex(appKey+contentMd5+urls+"application/json"+dateStr));
        Map<String, String> headers = new HashMap<>();
        headers.put("Content-type", "application/json");
        headers.put("X-Auth", xAuth);
        headers.put("Date", dateStr);
        headers.put("Content-Md5", contentMd5);

        String requestUrl = url + "/preview/v2/url?app_token=" + appToken +"&file_id=" + fileId + "&scene_id=" + sceneId;
        log.info("请求中台获取预览数据url的请求路径为："+requestUrl);
        log.info("请求中台预览数据URL的请求头："+headers);
        String response = HttpUtil.createGet(requestUrl).addHeaders(headers).execute().body();
        log.info("获取到的预览数据 url 的返回值："+response);
        if (StrUtil.isEmpty(response)) {
            return R.fail("获取预览url数据为空");
        }
        JSONObject json = JSONUtil.parseObj(response);
        if (json.containsKey("url")) {
            return R.data(json.getStr("url"));
        } else {
            return R.fail("获取预览url出错");
        }
    }

    /**
     * 获取应用token
     */
   private String getToken(){
       String appToken = "";
       String urls = String.format("/auth/v1/app/inscope/token?app_id=%s&scope=%s", appId, "file_preview");
       String body = null;
       String contentMd5 = DigestUtil.md5Hex(Optional.<String>ofNullable(body).orElse(""));
       String dateStr = ZonedDateTime.now().format(DateTimeFormatter.RFC_1123_DATE_TIME.withZone(ZoneId.of("GMT")));
       String xAuth = String.format("WPS-3:%s:%s", appId, DigestUtil.sha1Hex(appKey+contentMd5+urls+"application/json"+dateStr));
       Map<String, String> headers = new HashMap<>();
       headers.put("Content-type", "application/json");
       headers.put("X-Auth", xAuth);
       headers.put("Date", dateStr);
       headers.put("Content-Md5", contentMd5);
       String requestUrl = url + urls;
       log.info("请求路径："+ requestUrl);
       log.info("请求头参数：" + headers);
       String response = HttpUtil.createGet(requestUrl).addHeaders(headers).execute().body();
       log.info("获取到的返回值："+response);
       if (StrUtil.isNotEmpty(response)) {
           JSONObject jsonObject = JSONUtil.parseObj(response);
           if (jsonObject.containsKey("token")) {
               Object obj = jsonObject.get("token");
               JSONObject json = JSONUtil.parseObj(obj);
               appToken = json.getStr("app_token");
           }
       }
       log.info("获取的token为："+appToken);
       return appToken;
   }
}
