package com.search.supersearch.service;

import co.elastic.clients.elasticsearch.ElasticsearchClient;
import com.search.supersearch.pojo.DocsIndex;
import lombok.extern.slf4j.Slf4j;
import org.apache.tika.Tika;
import org.apache.tika.exception.TikaException;
import org.apache.tika.io.TikaInputStream;
import org.apache.tika.metadata.Metadata;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.net.URL;

/**
 * @author 喜欢番茄火锅
 * @date 2023/10/9 14:46
 */
@Service
@Slf4j
public class EsService {
    @Resource
    private ElasticsearchClient elasticsearchClient;

    public String saveFileByUrl(URL url, String index) throws IOException, TikaException {
        Tika tika = new Tika();
        Metadata metadata = new Metadata();
        InputStream stream = TikaInputStream.get(url, metadata);
//        Reader parse = tika.parse(stream,metadata);

        String s = tika.parseToString(url);
        DocsIndex docsIndex = new DocsIndex();
        docsIndex.setFileIndex("");
        docsIndex.setSystem("system");
        docsIndex.setTissueCoding("tissueCoding");
        docsIndex.setBusinessDataPrimaryKey("businessDataPrimaryKey" + url);
        docsIndex.setType("");
        docsIndex.setBusinessName("");
        docsIndex.setContent(s);
        docsIndex.setMetaAuthor("");
        docsIndex.setMetaDate("");
        docsIndex.setMetaModifier("");
        docsIndex.setMetaCreated("");
        docsIndex.setPathRoot("");
        docsIndex.setPathVirtual("");
        docsIndex.setPathReal("");
        docsIndex.setFileExtension("");
        docsIndex.setFileContentType(tika.detect(url));
        docsIndex.setFileCreated("");
        docsIndex.setFileLastModified("");
        docsIndex.setFileLastAccessed("");
        docsIndex.setFileIndexingDate("");
        docsIndex.setFilesize(0);
        docsIndex.setFilename(metadata.get("resourceName"));
        docsIndex.setFileUrl(url.toString());


        elasticsearchClient.index(i -> i.index(index).document(docsIndex));
        return s;
    }

    public String saveFileByFile(File file, String index) throws IOException, TikaException {
        log.info("解析文件为={}", file.getName());
        Tika tika = new Tika();
        tika.setMaxStringLength(-1);
        Metadata metadata = new Metadata();

        String s = tika.parseToString(file);
        DocsIndex docsIndex = new DocsIndex();
        docsIndex.setFileIndex("");
        docsIndex.setSystem("system");
        docsIndex.setTissueCoding("tissueCoding");
        docsIndex.setBusinessDataPrimaryKey("businessDataPrimaryKey" + file.getName());
        docsIndex.setType(index);
        docsIndex.setBusinessName("");
        docsIndex.setContent(s);
        docsIndex.setMetaAuthor("");
        docsIndex.setMetaDate("");
        docsIndex.setMetaModifier("");
        docsIndex.setMetaCreated("");
        docsIndex.setPathRoot("");
        docsIndex.setPathVirtual("");
        docsIndex.setPathReal("");
        docsIndex.setFileExtension("");
        docsIndex.setFileContentType(tika.detect(file));
        docsIndex.setFileCreated("");
        docsIndex.setFileLastModified("");
        docsIndex.setFileLastAccessed("");
        docsIndex.setFileIndexingDate("");
        docsIndex.setFilesize(0);
        docsIndex.setFilename(file.getName());
        docsIndex.setFileUrl(file.getPath());


        elasticsearchClient.index(i -> i.index(index).document(docsIndex));
        return s;
    }
}
