package com.tika.tika.controller;

import com.google.common.collect.Lists;
import com.tika.tika.dto.DocsDTO;
import com.tika.tika.dto.SearchDTO;
import com.tika.tika.pojo.SearchType;
import com.tika.tika.util.FileUtil;
import lombok.SneakyThrows;
import lombok.extern.slf4j.Slf4j;
import org.springblade.core.tool.api.R;
import org.springblade.core.tool.utils.StringUtil;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.elasticsearch.core.ElasticsearchOperations;
import org.springframework.data.elasticsearch.core.SearchHits;
import org.springframework.data.elasticsearch.core.mapping.IndexCoordinates;
import org.springframework.data.elasticsearch.core.query.Criteria;
import org.springframework.data.elasticsearch.core.query.CriteriaQuery;
import org.springframework.data.elasticsearch.core.query.HighlightQuery;
import org.springframework.data.elasticsearch.core.query.Query;
import org.springframework.data.elasticsearch.core.query.highlight.Highlight;
import org.springframework.data.elasticsearch.core.query.highlight.HighlightField;
import org.springframework.data.elasticsearch.core.query.highlight.HighlightParameters;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.stream.Collectors;

@RestController
@RequestMapping("/employeeInfo")
@Slf4j
public class DocsController {

    @Resource
    private ElasticsearchOperations elasticsearchOperations;

    @PostMapping("/search")
    @SneakyThrows
    public R<?> search(@RequestBody SearchDTO dto) {

        List<SearchType> searchType = FileUtil.getSearchType("系统对应es索引.json");
        ArrayList<String> strings = Lists.newArrayList();
        dto.getSearchType().stream().forEach(type -> {
            Objects.requireNonNull(searchType.stream().filter(v2 -> v2.getType().equals(type)).findAny().orElse(null))
                    .getIndexName().stream().forEach(v -> strings.add(v));
        });

        List<String> collect = strings.stream().distinct().collect(Collectors.toList());
        String[] array = new String[collect.size()];
        collect.toArray(array);

        String keyword = dto.getKeyword();
        Criteria criteria = Criteria.where("content").is(keyword).or("file.filename").is(keyword);

        HighlightParameters build = HighlightParameters.builder().withFragmentSize(100).build();
        // 高亮
        HighlightQuery highlightQuery = new HighlightQuery(new Highlight(build, Lists.newArrayList(new HighlightField("content"), new HighlightField("file.filename"))), DocsDTO.class);

        Query query = new CriteriaQuery(criteria, PageRequest.of(dto.getPage() == null ? 0 : dto.getPage() - 1, dto.getSize() == null ? 10 : dto.getSize()));

        if (StringUtil.isBlank(keyword)) {
            query = new CriteriaQuery(new Criteria(), PageRequest.of(dto.getPage() == null ? 0 : dto.getPage() - 1, dto.getSize() == null ? 10 : dto.getSize()));
        }
        query.setHighlightQuery(highlightQuery);
        SearchHits<Map> search = elasticsearchOperations.search(query, Map.class, IndexCoordinates.of(array));

//        SearchVO searchVO = new SearchVO();
//        searchVO.setTotalHits(search.getTotalHits());
//        List<Map> collect = search.getSearchHits().stream().map(v -> BeanUtil.beanToMap(v, false, false)).collect(Collectors.toList());
//        searchVO.setSearchHits(collect);
//        search.
        return R.data(search);
    }


}
