package com.tika.tika;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.json.JSONUtil;
import com.google.common.collect.Lists;
import com.tika.tika.controller.DocsController;
import com.tika.tika.dto.DocsDTO;
import com.tika.tika.dto.DocsDTO2;
import com.tika.tika.dto.SearchDTO;
import com.tika.tika.repository.DocsRepository;
import com.tika.tika.vo.SearchVO;
import lombok.SneakyThrows;
import org.junit.jupiter.api.Test;
import org.springblade.core.tool.api.R;
import org.springframework.beans.BeanUtils;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.data.elasticsearch.core.ElasticsearchOperations;
import org.springframework.data.elasticsearch.core.SearchHits;
import org.springframework.data.elasticsearch.core.mapping.IndexCoordinates;
import org.springframework.data.elasticsearch.core.query.*;
import org.springframework.data.elasticsearch.core.query.highlight.Highlight;
import org.springframework.data.elasticsearch.core.query.highlight.HighlightField;
import org.springframework.data.elasticsearch.core.query.highlight.HighlightParameters;
import org.springframework.test.context.ActiveProfiles;

import javax.annotation.Resource;
import java.util.List;
import java.util.Map;
import java.util.concurrent.atomic.AtomicBoolean;
import java.util.stream.Collectors;

@SpringBootTest
@ActiveProfiles("wr") //这里
class SuperSearchApplicationTests {


    @Resource
    private DocsRepository docsRepository;
    @Resource
    private ElasticsearchOperations elasticsearchOperations;
    @Resource
    private DocsController docsController;

    @Test
    @SneakyThrows
    void run7() {
        SearchDTO searchDTO = new SearchDTO();
        searchDTO.setSearchType(Lists.newArrayList(1));
        searchDTO.setKeyword("文字文稿");

        R<?> search = docsController.search(searchDTO);
        System.out.println(JSONUtil.toJsonStr(search));

    }

    /**
     * 保存 es 数据
     *
     * @param
     * @return
     * @author 王瑞
     * @date 2023/9/14
     */
    @Test
    @SneakyThrows
    void run6() {

        Criteria criteria = new Criteria();
        Query query = new CriteriaQuery(criteria);
        SearchHits<DocsDTO> search = elasticsearchOperations.search(query, DocsDTO.class, IndexCoordinates.of("test_docs1"));
        AtomicBoolean flag = new AtomicBoolean(true);
        search.iterator().forEachRemaining(v -> {
            DocsDTO content = v.getContent();
//            Map content = v.getContent();
            DocsDTO2 docsDTO2 = new DocsDTO2();
            BeanUtils.copyProperties(content, docsDTO2);
            IndexQuery indexQuery = new IndexQuery();
            indexQuery.setObject(content);

//            elasticsearchOperations.index(indexQuery,IndexCoordinates.of("test_docs2"));
            if (flag.get()) {
                elasticsearchOperations.save(docsDTO2, IndexCoordinates.of("test_docs3"));
                flag.set(false);
            } else {
                elasticsearchOperations.save(docsDTO2, IndexCoordinates.of("test_docs2"));
                flag.set(true);
            }
        });


    }

    @Test
    @SneakyThrows
    void run5() {

//        IndexOperations indexOperations = elasticsearchOperations.indexOps(DocsDTO2.class);
        elasticsearchOperations.indexOps(IndexCoordinates.of("test_docs2")).delete();
        elasticsearchOperations.indexOps(IndexCoordinates.of("test_docs3")).delete();
//        elasticsearchOperations.indexOps(IndexCoordinates.of("test_docs1")).delete();
//        System.out.println(withMapping);
    }

    @Test
    @SneakyThrows
    void run4() {
        String keyword = "网络安全风险";
//        Criteria criteria = Criteria.where("content").contains(keyword).or("file.filename").contains(keyword);
        Criteria criteria = Criteria.where("content").is(keyword);
//        Criteria criteria = Criteria.where("file.filename").is(keyword);
//                Criteria criteria = Criteria.where("content").is(keyword).or("file.filename").contains(keyword);

        // 高亮
//    SearchSourceBuilder.highlight().field("content").build();
        HighlightQuery highlightQuery = new HighlightQuery(new Highlight(
                HighlightParameters.builder().withFragmentSize(100).build(),
                Lists.newArrayList(new HighlightField("content"), new HighlightField("file.filename"))), DocsDTO.class);


        Query query = new CriteriaQuery(criteria);
        query.setHighlightQuery(highlightQuery);
        SearchHits<Map> search = elasticsearchOperations.search(query, Map.class, IndexCoordinates.of("test_docs1"));
//        System.out.println(JSONUtil.toJsonStr(search));

        SearchVO searchVO = new SearchVO();
        searchVO.setTotalHits(search.getTotalHits());
        List<Map> collect = search.getSearchHits().stream().map(v -> BeanUtil.beanToMap(v, true, false)).collect(Collectors.toList());
        searchVO.setSearchHits(collect);
//        System.out.println(JSONUtil.toJsonStr(search));
        System.out.println(JSONUtil.toJsonStr(searchVO));
    }

    @Test
    @SneakyThrows
    void run3() {
        Iterable<DocsDTO> all = docsRepository.findAll();
        all.iterator().forEachRemaining(v -> {
            System.out.println(v);
            System.out.println("===========================");
        });
        System.out.println(all);
    }

    @Test
    void contextLoads() {
        System.out.println(123);
    }

}
